;; Generated package description from tramp-theme.el  -*- no-byte-compile: t -*-
(define-package "tramp-theme" "0.2.0.20201201.173043" "Custom theme for remote buffers" '((emacs "24.1")) :url "https://elpa.gnu.org/packages/tramp-theme.html" :authors '(("Michael Albinus" . "michael.albinus@gmx.de")) :maintainer '("Michael Albinus" . "michael.albinus@gmx.de") :keywords '("convenience" "faces"))
